-- $Id: LeaseRenewer.lua 101374 2010-02-02 20:35:54Z rodrigoh $

local oil = require"oil"
local oop = require "loop.base"
local Timer = require "loop.thread.Timer"
local log = require "openbus.util.Log"

local tostring = tostring

---
--Objeto que  responsvel por renovar o lease junto a um Provider.
---
module ("openbus.lease.LeaseRenewer", oop.class)

---
--Constri o renovador de lease.
--
--@param lease O perodo de tempo entre as renovaes do lease.
--@param credential A credencial do membro.
--@param leaseProvider A entidade onde se deve renovar o lease.
--@param leaseExpiredCallback Instncia de LeaseExpiredCallback que ser
--       executada quando o lease expirar.
--
--@return O renovador de lease.
---
function __init(self, lease, credential, leaseProvider, leaseExpiredCallback)
  log:lease("Lease set to "..lease)
  return oop.rawnew(self,
    { lease = lease,
      credential = credential,
      provider = leaseProvider,
      leaseExpiredCallback = leaseExpiredCallback, -- opcional
    })
end

---
--Define por quanto tempo o lease ser vlido.
--
--@param lease O tempo de validade do lease.
---
function setLease(self, lease)
  self.lease = lease
  log:lease("Lease set to "..lease)
end

---
--Obtm o tempo de validade do lease.
--
--@return O tempo de validade do lease.
---
function getLease(self)
  return self.lease
end

---
--Define o provedor responsvel pelo lease, ou seja, onde o lease deve ser
--renovado.
--
--@provider O provedor.
---
function setProvider(self, provider)
  self.provider = provider
end

---
--Obtm o provedor do lease.
--
--@return O provedor do lease.
---
function getProvider(self)
  return self.provider
end

---
--Define o observador do lease.
--
--@lec o Observador.
---
function setLeaseExpiredCallback(self, lec)
  self.leaseExpiredCallback = lec
end

---
--Inicia a execuo do renovador de lease.
---
function startRenew(self)
  if not self.timer then
    -- Aloca uma "thread" para a renovao do lease
    local timer = Timer{
      scheduler = oil.tasks,
      rate = self.lease,
    }
    function timer.action(timer)
      local provider = self:getProvider()
      local success, granted, newlease  =
        oil.pcall(provider.renewLease, provider, self.credential)
      if not success then
        local exception = granted
        if exception[1] == "IDL:omg.org/CORBA/NO_PERMISSION:1.0" then
          log:lease("Sem permisso para renovar o lease.")
          granted = false
        else
          -- Quando ocorre falha no acesso ao provedor, tenta renovar com
          -- uma freqncia maior.
          log:error("Falha na acessibilidade ao provedor do lease.\nMensagem: "
              ..tostring(granted))
          return
        end
      end
      if not granted then
        log:lease("Lease no renovado.")
        timer:disable()
        if self.leaseExpiredCallback then
          self.leaseExpiredCallback:expired()
        end
        return
      end
      if timer.rate ~= newlease then
        timer.rate = newlease
        self:setLease(newlease)
      end
    end
    self.timer = timer
  end
  self.timer.rate = self.lease
  self.timer:enable()
end

---
--Termina a execuo do renovador.
---
function stopRenew(self)
  self.timer:disable()
end
