/**
* \mainpage API - Openbus Orbix C++
* \file openbus.h
*/

#ifndef OPENBUS_H_
#define OPENBUS_H_

#include "verbose.h"
#include "stubs/access_control_service.hh"
#include "services/RegistryService.h"
#include "stubs/session_service.hh"

#include "openbus/common/ORBInitializerImpl.h"
#include <ComponentBuilderOrbix.h>

#include <omg/orb.hh>
#include <it_ts/thread.h>
#include <it_ts/mutex.h>

#include <stdexcept>
#include <set>

using namespace openbusidl::acs;
IT_USING_NAMESPACE_STD

/**
* \brief Stubs dos servios bsicos.
*/
namespace openbusidl {

/**
* \brief Stub do servio de acesso.
*/
  namespace acs {

  /**
  * \class Credential
  * \brief Credencial de acesso ao barramento.
  */

  }
}

/**
* \brief openbus
*/
namespace openbus {

/**
* \brief Falha no processo de login, ou seja, o par nome de usurio e senha no
* foi validado.
*/
  class LOGIN_FAILURE : public runtime_error {
    public:
      LOGIN_FAILURE(const string& msg = "") : runtime_error(msg) {}
  };

/**
* \brief Falha no mecanismo de autenticao por certificado digital.
* Algumas possveis causas:
*  + No foi possvel obter o desafio.
*  + Falha na manipulao de uma chave privada ou pblica.
*  + Falha na manipulao de um certificado.
*/
  class SECURITY_EXCEPTION : public runtime_error {
    public:
      SECURITY_EXCEPTION(const string& msg = "") : runtime_error(msg) {}
  };

/**
* \brief No h conexo estabelecida com nenhum barramento.
*/
  class NO_CONNECTED : public runtime_error {
    public:
      NO_CONNECTED(const string& msg = "") : runtime_error(msg) {}
  };

/**
* \brief No  possvel obter o servio de sesso no barramento em uso.
*/
  class NO_SESSION_SERVICE : public runtime_error {
    public:
      NO_SESSION_SERVICE(const string& msg = "") : runtime_error(msg) {}
  };

  /**
  * \brief Representa um barramento.
  */
  class Openbus {
    public:
      class LeaseExpiredCallback;

    private:

    /**
    * Mutex. 
    */
      static IT_Mutex mutex;

    /**
    * A instncia nica do barramento.
    */
      static Openbus* bus;

    /**
    * Parmetro argc da linha de comando. 
    */
      int _argc;

    /**
    * Parmetro argv da linha de comando. 
    */
      char** _argv;
      
    /**
    * Ponteiro para o stub do servio de acesso.
    */
      openbusidl::acs::IAccessControlService_var iAccessControlService;

    /**
    * Ponteiro para o stub do servio de registro.
    */
      openbusidl::rs::IRegistryService* iRegistryService;

    /**
    * Ponteiro para o stub do servio de sesso.
    */
      openbusidl::ss::ISessionService* iSessionService;

    /**
    * Ponteiro para a faceta ILeaseProvider. 
    */
      openbusidl::acs::ILeaseProvider_var iLeaseProvider;

    /**
    * Inicializador do ORB. 
    */
      static common::ORBInitializerImpl* ini;

    /**
    * ORB. 
    */
      CORBA::ORB_var orb;

    /**
    * POA.
    */
      PortableServer::POA_var poa;

    /**
    * Fbrica de componentes SCS. 
    */
      scs::core::ComponentBuilder* componentBuilder;

    /**
    * Gerenciador do POA. 
    */
      PortableServer::POAManager_var poa_manager;

    /**
    * Servio de registro. 
    */
      services::RegistryService* registryService;

    /**
    * Intervalo de tempo que determina quando a credencial expira. 
    */
      Lease lease;

    /**
    * Credencial de identificao do usurio frente ao barramento. 
    */
      openbusidl::acs::Credential_var credential;

    /**
    * Mquina em que est o barramento. 
    */
      string hostBus;

    /**
    * A porta da mquina em que se encontra o barramento.
    */
      unsigned short portBus;

    /**
    * Possveis estados para a conexo. 
    */
      enum ConnectionStates {
        CONNECTED,
        DISCONNECTED
      };

    /**
    * Indica o estado da conexo. 
    */
      ConnectionStates connectionState;

    /**
    * Intervalo de tempo que determina quando que a credencial ser renovada.
    */
      unsigned long timeRenewing;

    /**
    * Especifica se o tempo de renovao de credencial  fixo.
    */
      bool timeRenewingFixe;

    /**
    * Trata os parmetros de linha de comando.
    */
      void commandLineParse(
        int argc,
        char** argv);

    /**
    * Inicializa um valor default para a mquina e porta do barramento. 
    */
      void initialize();

    /**
    * Cria implicitamente um ORB e um POA. 
    */
      void createOrbPoa();

    /**
    * Registra os interceptadores cliente e servidor. 
    */
      void registerInterceptors();

    /**
    * Cria um estado novo. 
    */
      void newState();
      
    /**
    * Cria o proxy para o servio de acesso.
    */
      void createProxyToIAccessControlService();

    /**
    * Desconexo local.
    */
      void localDisconnect();

    /**
    * Cria o objeto registryService.
    */
      services::RegistryService* setRegistryService();

      IT_Thread renewLeaseIT_Thread;

    /**
    * Thread responsvel pela renovao da credencial do usurio que est 
    * logado neste barramento.
    */
      class RenewLeaseThread : public IT_ThreadBody {
        private:
        /**
        * Callback registrada para a notificao da 
        * expirao do lease.
        */
          LeaseExpiredCallback* leaseExpiredCallback;
        public:
          RenewLeaseThread();
          void setLeaseExpiredCallback(LeaseExpiredCallback* obj);
          void* run();
      };
      friend class Openbus::RenewLeaseThread;

    /**
    * Thread responsvel pela renovao de credencial.
    */
      static RenewLeaseThread* renewLeaseThread;

      Openbus();

    public:

    #ifdef VERBOSE
      static Verbose* verbose;
    #endif

      ~Openbus();

    /**
    * Fornece a nica instncia do barramento.
    *
    * @return Openbus
    */
      static Openbus* getInstance();

    /**
    * Inicializa uma referncia a um barramento.
    *
    * Um ORB e um POA so criados implicitamente.
    * A fbrica de componentes SCS  criada.
    * Os argumentos Openbus de linha de comando (argc e argv) so tratados.
    * Parmetros de linha de comando: 
    *   "-OpenbusHost": Host do barramento.
    *   "-OpenbusPort": Porta do barramento.
    *   "-TimeRenewing": Tempo em milisegundos de renovao da credencial.
    *
    * @param[in] argc
    * @param[in] argv
    */
      void init(
        int argc,
        char** argv);

    /**
    * Inicializa uma referncia a um barramento.
    *
    * Um ORB e um POA so criados implicitamente.
    * A fbrica de componentes SCS  criada.
    * Os argumentos Openbus de linha de comando (argc e argv) so tratados.
    * Parmetros de linha de comando: 
    *   "-OpenbusHost": Mquina em que se encontra o barramento.
    *   "-OpenbusPort": Porta do barramento.
    *   "-TimeRenewing": Tempo em milisegundos de renovao da credencial.
    *
    * @param[in] argc
    * @param[in] argv
    * @param[in] host Mquina em que se encontra o barramento.
    * @param[in] port A porta da mquina em que se encontra o barramento.
    */
      void init(
        int argc,
        char** argv,
        char* host,
        unsigned short port);

    /**
    * Informa o estado de conexo com o barramento.
    *
    * @return true caso a conexo esteja ativa, ou false, caso
    * contrrio.
    */
      bool isConnected();

    /** 
    *  Termination Handler disponvel para a classe IT_TerminationHandler()
    *
    *  @param signalType
    */
      static void terminationHandlerCallback(long signalType);

    /**
    *  Retorna o ORB utilizado.
    *  @return ORB
    */
      CORBA::ORB* getORB();

    /**
    *  Retorna o RootPOA.
    *
    *  OBS: A chamada a este mtodo ativa o POAManager.
    *
    *  @return POA
    */
      PortableServer::POA* getRootPOA();

    /**
    * Retorna a fbrica de componentes. 
    * @return Fbrica de componentes
    */
      scs::core::ComponentBuilder* getComponentBuilder();

    /**
    * Retorna a credencial interceptada pelo interceptador servidor. 
    * @return Credencial. \see openbusidl::acs::Credential
    */
      Credential_var getInterceptedCredential();

    /**
    * Retorna o servio de acesso. 
    * @return Servio de acesso
    */
      openbusidl::acs::IAccessControlService* getAccessControlService();

    /**
    * Retorna o servio de registro. 
    * @return Servio de registro
    */
      services::RegistryService* getRegistryService();

    /**
    * Retorna o servio de sesso. 
    * @return Servio de sesso.
    */
      openbusidl::ss::ISessionService* getSessionService() 
        throw(NO_CONNECTED, NO_SESSION_SERVICE);

    /**
    * Retorna a credencial de identificao do usurio frente ao barramento. 
    * @return credencial
    */
      Credential* getCredential();

    /**
    * Define uma credencial a ser utilizada no lugar da credencial corrente. 
    * til para fornecer uma credencial com o campo delegate preenchido.
    * 
    * @param[in] credential Credencial a ser utilizada nas requisies a serem
    *   realizadas.
    */
      void setThreadCredential(Credential* credential);

    /**
    * Representa uma callback para a notificao de que um lease expirou.
    */
      class LeaseExpiredCallback {
        public:
          virtual void expired() = 0;
      };

    /**
    * Registra uma callback para a notificao de que o lease da credencial
    * de identificao do usurio, frente ao barramento, expirou.
    *
    * @param[in] A callback a ser registrada.
    * @return True se a callback foi registrada com sucesso, ou false 
    * se a callback j estava registrada.
    */
      void addLeaseExpiredCallback(
        LeaseExpiredCallback* leaseExpiredCallback);

    /**
    * Remove uma callback previamente registra para a notificao de lease 
    * expirado.
    *
    * @param[in] A callback a ser removida.
    * @return True se a callback foi removida com sucesso, ou false 
    * caso contrrio.
    */
      void removeLeaseExpiredCallback();

    /**
    *  Realiza uma tentativa de conexo com o barramento.
    *
    *  @param[in] user Nome do usurio.
    *  @param[in] password Senha do usurio.
    *  @throw LOGIN_FAILURE O par nome de usurio e senha no foram validados.
    *  @throw CORBA::SystemException Alguma falha de comunicao com o 
    *    barramento ocorreu.
    *  @return  Se a tentativa de conexo for bem sucedida, uma instncia que 
    *    representa o servio  retornada.
    */
      services::RegistryService* connect(
        const char* user,
        const char* password)
        throw (CORBA::SystemException, LOGIN_FAILURE);

    /**
    *  Realiza uma tentativa de conexo com o barramento utilizando o
    *    mecanismo de certificao para o processo de login.
    *
    *  @param[in] entity Nome da entidade a ser autenticada atravs de um
    *    certificado digital.
    *  @param[in] privateKeyFilename Nome do arquivo que armazena a chave
    *    privada do servio.
    *  @param[in] ACSCertificateFilename Nome do arquivo que armazena o
    *    certificado do servio.
    *  @throw LOGIN_FAILURE O par nome de usurio e senha no foram validados.
    *  @throw CORBA::SystemException Alguma falha de comunicao com o 
    *    barramento ocorreu.
    *  @throw SECURITY_EXCEPTION Falha no mecanismo de autenticao por 
    *    certificado digital.
    *    Algumas possveis causas:
    *     + No foi possvel obter o desafio.
    *     + Falha na manipulao de uma chave privada ou pblica.
    *     + Falha na manipulao de um certificado.
    *       entidade ou do certificado do ACS.
    *  @return  Se a tentativa de conexo for bem sucedida, uma instncia que 
    *    representa o servio  retornada.
    */
      services::RegistryService* connect(
        const char* entity,
        const char* privateKeyFilename,
        const char* ACSCertificateFilename)
        throw (CORBA::SystemException, LOGIN_FAILURE, SECURITY_EXCEPTION);

    /**
    *  Desfaz a conexo atual.
    *
    *  @return Caso a conexo seja desfeita, true  retornado, caso contrrio,
    *  o valor de retorno  false.
    */
      bool disconnect();

    /**
    * Loop que processa requisies CORBA. [execuo do orb->run()]. 
    */
      void run();

    /**
    * Finaliza a execuo do ORB.
    *
    * @param[in] bool force Se a finalizao deve ser forada ou no.
    */
      void finish(bool force);

  };
}

#endif

